#ifndef VECTOR_H
#define VECTOR_H

#include <math.h>
#include <GL/glut.h>

class Vector {
	float length; // Length of the Vector instance

	public:
        float px, py, pz; // position of the Vector in world coordinates
	    float x, y, z; // mathematical Vector values

	Vector(float pxIn, float pyIn, float pzIn, float xIn, float yIn, float zIn);

    /* Constructor for 3D vectors.  Accepts the x, y, and z vector components 
     * of the vector.  Defaults the vector position to the origin.
     */
    Vector(float xIn, float yIn, float zIn);

	Vector(float pxIn, float pyIn, float xIn, float yIn);
    Vector(float pxIn, float xIn);
	Vector() : px(0), py(0), pz(0), x(0), y(0), z(0), length(0) {}

	/* Computes the dot product of the two given Vectors a and b.
	 * Returns the result as a float.
	 */
	static float dotProduct(Vector a, Vector b);

    /* Computes the cross product of the given Vectors.  Returns a Vector as 
     * the result.
     * 
     * WARNING: This assumes given Vectors have positions at the origin.  
     * Therefore the resulting Vector will also have its position at the origin.
     */
    static Vector crossProduct(Vector a, Vector b);

    /**
     * Mathematical vector addition.  The vector components are summed together.
     * The new Vector's vector components and length will be be the result of
     * operation.  The position of the vector is at the origin.
     */
    Vector operator+(Vector rhs);

    /**
     * Same as operator+ except that the current Vector object is assigned to
     * result of the mathematical vector addition.  Returns itself so that
     * future expressions may be chained.
     */
    Vector operator+=(Vector rhs);

    /**
     * Performs a mathematical scalar multiplication on the Vector object. The
     * resulting Vector is returned.  The length will be updated.
     */
    Vector operator*(float rhs);

    /**
     * Performs a mathematical scalar mulitplication (division in this case)
     * on the Vector object.  The resulting Vector is returned.  The length
     * will be updated.
     */
    Vector operator/(float rhs);

    /**
     * Assigns the current Vector object to the Vector on its right hand side.
     * Copies all instance variables from the rhs.  Returns the result of the
     * assignment so that future expressions can be chained.
     */
    Vector operator=(Vector rhs);

	float getLength();

    /* Computes the perpendicular vector relative to this vector instance.  The
     * starting position of the perpendicular vector is taken to be the midpoint
     * of this vector (when drawn out).
     */
    Vector computePerpendicular();

	/* Plots/draws the vector in the world coordinate frame.  The vector head
	 * will be drawn to distinguish it from other lines.  The default coloring
	 * is used.
	 */
	void plot();

	/* Normalizes the vector.  Updates the length to reflect this.  Returns the
     * normalized vector so that future expressions can chain.
	 */
	Vector normalize();

    /**
     * Writes out the Vector object to array form.  Only the Vector's math
     * vector components are written out as: [x, y, z].  Caller *OWNS* the
     * object.
     */
    float* toArray();
};

#endif
